;*******************************************************************************
;
;                               PIC2DHTI2C
;
; DrG 2016
;
;*******************************************************************************
;
;
; PIC12F1840 to multiple DHT22 and
; PIC12F1840 as a slave I2C device
;
; This software is offered strictly as is with no warranties whatsoever.
; Use it at your own risk.
;
; The program uses I2C code based on AN734C and modified for the 12F1840
; Whatever copyrights and liscensing agreements exist for using that
; code are still in effect so I have left the verbage from the AN intact
;*******************************************************************************
;-------------------------------------------------------------------------------

	list		p=12F1840      ; list directive to define processor
	#include	<p12F1840.inc> ; processor specific variable definitions
    errorlevel  -302           ; no warnings about non bank 0 regs
    errorlevel  -303           ; no warnings about prog word too large
;
; CONFIGS
;_FOSC_INTOSC    INTOSC oscillator (16mHz:) I/O function on CLKIN pin
;_WDTE_ON        WDT enabled
;_PWRTE_OFF      PWRT disabled <--
;_MCLRE_OFF      MCLR/VPP pin function is digital input <--
;_CP_OFF         Program memory code protection is disabled <--
;_CPD_OFF        Data memory code protection is disabled <--
;_BOREN_OFF      Brown-out Reset disabled
;_CLKOUTEN_OFF   CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
;_IESO_ON        Internal/External Switchover mode is enabled
;_FCMEN_ON       Fail-Safe Clock Monitor is enabled
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_ON & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_ON & _FCMEN_ON
;_WRT_OFF        Write protection off
;_PLLEN_ON       PLL enabled
;_STVREN_OFF     Stack Overflow or Underflow will not cause a Reset
;_BORV_19        Brown-out Reset Voltage (Vbor), low trip point selected.
;_LVP_OFF        High-voltage on MCLR/VPP must be used for programming
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF
;
#define I2C_ADDRESS 	0x32				; Slave address
#define	RX_ELEMENTS		0x20             	; number of allowable array elements, in this case 32
;These are the data bits for the three DHT22s
; RA1=SCL / RA2=SDA / RA3=MCLR input only
DQ1      EQU RA0                             ; DHT22_1 DQ is on RA0
DQ2      EQU RA4                             ; DHT22_2 DQ is on RA4
DQ3      EQU RA5                             ; DHT22_3 DQ is on RA5
;------------------------------------ variables--------------------------------------------------
; NOTE these are put in common RAM so we don't have to BANKSEL but
; remember we only have 16 bytes
    cblock 0x70                             ; set up in shared memory for easy access
		INDEX_I2C							; index used to point to array location
		FIRST								; used to determine if it is first data byte
        d0                                  ; delay counters
        d1
        d2
        bytecnt                             ; byte counter
        bitcnt                              ; bit counter
    endc
; I2C_ARRAY BYTES
; [0] = request byte set by the master. Initialized to 0 by PIC
;
; Slave [PIC] sets [0]
;            [0x00] start/reset value
;            [0x01] successful execution of the command
;            [0xf1] unspecified error or bad / unimplemented command
; Master [Galileo] sets [0] to request actions
;            [0x02] Read DHT #1
;            [0x03] Read DHT #2
;            [0x04] Read DHT #3
;            [0x10] RESET PIC
; [1]-[5]   will hold DHT #1 data
; [6]-[10]  will hold DHT #2 data
; [11]-[15] will hold DHT #3 data
; [16]-[31] free and available
;-------------------
; 5 bytes of DHT info are as follows:
; DHT data
; RH integral
; RH decimal
; Temp integral
; Temp decimal
; checksum is the sum of all four bytes AND 255
; -------------------
;
; I2C_ARRAY will end up at 0x20 so watch your banks!
; the irq will save the BSR for the I2C/SSP bank, just watch the rest
    udata
I2C_ARRAY   res RX_ELEMENTS              ; array to write to, array memory size
;
;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 12F1840 has 256 bytes of non-volatile EEPROM, starting at address 0xF000
;
;------------------------------------------------------------------------------
; just for s & g
DATAEE    ORG  0xF000
    DE    "DrGee"  ; Place 'D' 'r' 'G' 'e' 'e'  at address 0,1,2,3,5
;
;------------------------------------------------------------------------------
; MACROS
;-----------------------------------------------------------------------------------------------
; The macro LOADFSR loads FSR1 with the I2C address and the I2C index value to read or write to
; and makes code easier to read.
LOADFSR macro 	ADDRESS,INDEX 				; ADDRESS = I2C_ARRAY, INDEX = INDEX_I2C
		movlw 	ADDRESS 					; load address
		addwf	INDEX,W						; add the index value to determine location in array
		movwf 	FSR1L						; load FSR1L with pointer info
		clrf	FSR1H
		endm
;-----------------------------------------------------------------------------------------------
;
;----------------------------------- start vector ------------------------------
 		ORG    	0x0000          			; reset vector
    	goto   	Main                    	; jump to main
;----------------------------------- interrupt vector --------------------------
    	ORG    	0x0004          			; interrupt vector
		banksel	PIR1
		btfss 	PIR1,SSP1IF 				; Is this a SSP interrupt?
		goto 	BUS_COLL 					; if not, bus collision int occurred
    	banksel	SSPSTAT
		btfsc	SSPSTAT,2					; is it a master read:
		goto	READ						; if so go here
		goto	WRITE						; if not, go here
;
;-------------------------------------------------------------------------------
;	                              MAIN
;-------------------------------------------------------------------------------
		ORG		0x0040        				; start of Flash memory
Main
		call	INITIALIZE					; set up uC
        movlw   0x00
        movwf   I2C_ARRAY                   ; clear command array[0]
LOOP
		clrwdt                              ; bump the WDT
;
; commands / status at I2C_ARRAY[0]
; first check for acceptable status values and just loop until a command request
        movf	I2C_ARRAY,0     ; array[0] to W
        iorlw   0x0             ; array[0]=0x00 no command / reset condition
        btfsc   STATUS,Z
        goto    LOOP
        movlw   0x01            ; array[0]=0x01 no command [last command competed ok)
        subwf   I2C_ARRAY,W
        btfsc   STATUS,Z
        goto    LOOP
        movlw   0xf1            ; array[0]=0xf1 bad command on the last operation
        subwf   I2C_ARRAY,W
        btfsc   STATUS,Z
        goto    LOOP
; check for legitimate command requests here
        movlw   0x02            ; array[0]=0x02 - read DHT #1
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; Check if zero..
        goto    ReadDHT1
        movlw   0x03            ; array[0]=0x3 - read DHT #2
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; Check if zero..
        goto    ReadDHT2
        movlw   0x04            ; array[0]=0x04 - read DHT #3
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; Check if zero..
        goto    ReadDHT3
        movlw   0x10            ; array[0]=0x10 - RESET PIC
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; Check if zero..
        goto    ResetPIC
; fall through to a bad cmd if nothing legitimate was found
;----- BADCMD
BADCMD
; the command byte was not recognized - put 0xf1 in [0]
        movlw   0xf1                        ; bad command code
        movwf   I2C_ARRAY                   ; into [0]
        goto    LOOP
CMDDONE
; successful execution of the command
; indicate success by setting [0]=1
        movlw   0x01
        movwf   I2C_ARRAY
        goto    LOOP
;---------------------------------------------------------------------------
;Read the first DHT-22 and save the 5 data bytes into DHT
ReadDHT1
; set up for the transfer to I2C_ARRAY +1 to +5
    movlw   0x05            ; bytes from the DHT
    movwf   bytecnt
    movlw   0x08            ; bits
    movwf   bitcnt
    movlw   (I2C_ARRAY+1)   ; point to start of buffer in ARRAY
    movwf 	FSR0L			; load FSR0L with pointer info
	clrf	FSR0H
;initialize the DHT
Init_HU_1
; make data pin output and set it low
   banksel LATA
   bcf     LATA,DQ1
   banksel TRISA
   bcf     TRISA,DQ1
; delay 18 ms
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_1
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_1
	goto  $+1
; bring data line high for 30 us
   banksel LATA
   bsf     LATA,DQ1
; delay for 30 us (20-40 us as per data sheet)
	movlw 0x27
	movwf d0
D30us_1
	decfsz d0, f
	goto D30us_1
	goto  $+1
; now it's the DHT's turn to talk...
; set data line for input
    banksel LATA
    bcf     LATA,DQ1
    banksel TRISA
    bsf     TRISA,DQ1
; the DHT should set it low for 80us but check and wait until it is low
wlow_1
    banksel PORTA
    btfsc   PORTA,DQ1
    goto    wlow_1
; the DHT should next set the data line high for 80us
whigh_1
    btfss   PORTA,DQ1
    goto    whigh_1
; now wait for it to go low again, marking the begining of the start bit
wlow2_1
    btfsc   PORTA,DQ1
    goto    wlow2_1
; here is the start bit - a 50us low on the data line followed
; by 26-28 us high if a "0" or a 70 us high if a "1"
start_bit_1
; wait for the start bit time to end
whigh2_1
    btfss   PORTA,DQ1
    goto    whigh2_1
; and then delay for 40 us
	movlw 0x35
	movwf d0
D40us_1
	decfsz d0, f
	goto D40us_1
; now read the data line to determine if it is 1 or was a 0
; note we are still in the PORTA bank
read_bit_1
    btfsc   PORTA,DQ1
    goto    one_bit_1
; the bit was a "0" and it is over and the HU is in the start bit time
; with ~24 us left - process the bit and jump to wait for high2
    bcf     STATUS, C
    rlf     INDF0,1
; dec bit counter
dec_bit_1
    decfsz    bitcnt,f
    goto      no_bump_1
;reset bit_count and dec bytcount
    movlw   0x08
    movwf   bitcnt
;point to the next dataum holder
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_1
; here we have finished all 5 bytes of data so transmission is finished
    goto fin_trans_1
no_bump_1
    goto    whigh2_1
one_bit_1
; the bit was a "1" and we are still in the bit time - wait for it to end first
; then process the bit and check for bumps and fall through to wait_high2
wait_for_end_1
    btfsc   PORTA,DQ1
    goto    wait_for_end_1
; process the bit
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_1
;
fin_trans_1
;we have read the HU and all 5 bytes of data are loaded into ARRAY[1]-[5]
    goto    CMDDONE
;
;Read the second DHT-22 and save the 5 data bytes into DHT
ReadDHT2
; set up for the transfer to I2C_ARRAY +6 to +10
    movlw   0x05            ; bytes from the DHT
    movwf   bytecnt
    movlw   0x08            ; bits
    movwf   bitcnt
    movlw   (I2C_ARRAY+6)   ; point to start of buffer in ARRAY
    movwf 	FSR0L			; load FSR0L with pointer info
	clrf	FSR0H
;initialize the DHT
Init_HU_2
; make data pin output and set it low
   banksel LATA
   bcf     LATA,DQ2
   banksel TRISA
   bcf     TRISA,DQ2
; delay 18 ms
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_2
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_2
	goto  $+1
; bring data line high for 30 us
   banksel LATA
   bsf     LATA,DQ2
; delay for 30 us (20-40 us as per data sheet)
	movlw 0x27
	movwf d0
D30us_2
	decfsz d0, f
	goto D30us_2
	goto  $+1
; now it's the DHT's turn to talk...
; set data line for input
    banksel LATA
    bcf     LATA,DQ2
    banksel TRISA
    bsf     TRISA,DQ2
; the DHT should set it low for 80us but check and wait until it is low
wlow_2
    banksel PORTA
    btfsc   PORTA,DQ2
    goto    wlow_2
; the DHT should next set the data line high for 80us
whigh_2
    btfss   PORTA,DQ2
    goto    whigh_2
; now wait for it to go low again, marking the begining of the start bit
wlow2_2
    btfsc   PORTA,DQ2
    goto    wlow2_2
; here is the start bit - a 50us low on the data line followed
; by 26-28 us high if a "0" or a 70 us high if a "1"
start_bit_2
; wait for the start bit time to end
whigh2_2
    btfss   PORTA,DQ2
    goto    whigh2_2
; and then delay for 40 us
	movlw 0x35
	movwf d0
D40us_2
	decfsz d0, f
	goto D40us_2
; now read the data line to determine if it is 1 or was a 0
; note we are still in the PORTA bank
read_bit_2
    btfsc   PORTA,DQ2
    goto    one_bit_2
; the bit was a "0" and it is over and the HU is in the start bit time
; with ~24 us left - process the bit and jump to wait for high2
    bcf     STATUS, C
    rlf     INDF0,1
; dec bit counter
dec_bit_2
    decfsz    bitcnt,f
    goto      no_bump_2
;reset bit_count and dec bytcount
    movlw   0x08
    movwf   bitcnt
;point to the next dataum holder
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_2
; here we have finished all 5 bytes of data so transmission is finished
    goto fin_trans_2
no_bump_2
    goto    whigh2_2
one_bit_2
; the bit was a "1" and we are still in the bit time - wait for it to end first
; then process the bit and check for bumps and fall through to wait_high2
wait_for_end_2
    btfsc   PORTA,DQ2
    goto    wait_for_end_2
; process the bit
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_2
;
fin_trans_2
;we have read the HU and all 5 bytes of data are loaded into ARRAY[6]-[10]
    goto    CMDDONE
;
ReadDHT3
;Read the third DHT-22 and save the 5 data bytes into DHT
;
; set up for the transfer to I2C_ARRAY +11 to +15
    movlw   0x05            ; bytes from the DHT
    movwf   bytecnt
    movlw   0x08            ; bits
    movwf   bitcnt
    movlw   (I2C_ARRAY+0x0b); point to start of buffer in ARRAY
    movwf 	FSR0L			; load FSR0L with pointer info
	clrf	FSR0H
;initialize the DHT
Init_HU_3
; make data pin output and set it low
    banksel LATA
    bcf     LATA,DQ3
    banksel TRISA
    bcf     TRISA,DQ3
; delay 18 ms
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_3
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_3
	goto  $+1
; bring data line high for 30 us
   banksel LATA
   bsf     LATA,DQ3
; delay for 30 us (20-40 us as per data sheet)
	movlw 0x27
	movwf d0
D30us_3
	decfsz d0, f
	goto D30us_3
	goto  $+1
; now it's the DHT's turn to talk...
; set data line for input
    banksel LATA
    bcf     LATA,DQ3
    banksel TRISA
    bsf     TRISA,DQ3
; the DHT should set it low for 80us but check and wait until it is low
wlow_3
    banksel PORTA
    btfsc   PORTA,DQ3
    goto    wlow_3
; the DHT should next set the data line high for 80us
whigh_3
    btfss   PORTA,DQ3
    goto    whigh_3
; now wait for it to go low again, marking the begining of the start bit
wlow2_3
    btfsc   PORTA,DQ3
    goto    wlow2_3
; here is the start bit - a 50us low on the data line followed
; by 26-28 us high if a "0" or a 70 us high if a "1"
start_bit_3
; wait for the start bit time to end
whigh2_3
    btfss   PORTA,DQ3
    goto    whigh2_3
; and then delay for 40 us
	movlw 0x35
	movwf d0
D40us_3
	decfsz d0, f
	goto D40us_3
; now read the data line to determine if it is 1 or was a 0
; note we are still in the PORTA bank
read_bit_3
    btfsc   PORTA,DQ3
    goto    one_bit_3
; the bit was a "0" and it is over and the HU is in the start bit time
; with ~24 us left - process the bit and jump to wait for high2
    bcf     STATUS, C
    rlf     INDF0,1
; dec bit counter
dec_bit_3
    decfsz  bitcnt,f
    goto    no_bump_3
;reset bit_count and dec bytcount
    movlw   0x08
    movwf   bitcnt
;point to the next dataum holder
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_3
; here we have finished all 5 bytes of data so transmission is finished
    goto    fin_trans_3
no_bump_3
    goto    whigh2_3
one_bit_3
; the bit was a "1" and we are still in the bit time - wait for it to end first
; then process the bit and check for bumps and fall through to wait_high2
wait_for_end_3
    btfsc   PORTA,DQ3
    goto    wait_for_end_3
; process the bit
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_3
;
fin_trans_3
;we have read the HU and all 5 bytes of data are loaded into ARRAY[11]-[15]
    goto    CMDDONE
;
;---------------------------------------------------------------------------
;
;----- Reset PIC
; do the software equivalent of a hardware reset
; this is just an emergency bailout that should not need to be used
ResetPIC
    RESET
;------------------------------------- end main ------------------------------------------------
;
;-----------------------------------------------------------------------------------------------
;	Initialize: Sets up register values
;-----------------------------------------------------------------------------------------------
INITIALIZE
;uC set up
; configure the oscillator to 16 mHz
;    movlw   b'01111000'
;              1        4x PLL is enabled
;               1111    16 MHz Internal Osc
;                   x   n/i
;                    1  internal osc block
;                     0  from config
;                     x n/i when bit 1=1
;NOTE 4x PLL is enabled which requires a CONFIG
		banksel	OSCCON
		movlw	b'01111010'					; Internal OSC @ 16MHz, Internal OSC block selected
		movwf	OSCCON                      ; 4xPLL is enabled (set in config word)
		movlw	b'11010111'                 ; WPU disabled, INT on rising edge, FOSC/4,
		movwf	OPTION_REG					; Prescaler assigned to TMR0, prescaler rate 1:256
        movlw   b'00011011'                 ; 8s wdt and swdten on
		movwf	WDTCON
; set all I/O to inputs
        banksel PORTA
        clrf    PORTA       ; init porta
        banksel LATA
        clrf    LATA        ; data latch
        banksel ANSELA
        clrf    ANSELA      ; NOTE setting to digital I/O  ***********
        banksel TRISA
; set all to input
        movlw   b'111111'
        movwf   TRISA
		clrf	INDEX_I2C                   ; Clear the index pointer
		clrf	FIRST                       ; Clear FIRST
		bsf		FIRST,0                     ; set bit 0 in FIRST
;I2C set up
		banksel	SSPSTAT
		bsf		SSPSTAT,SMP					; Slew rate control disabled for standard speed mode
		movlw	b'00110110'					; Enable serial port, I2C slave mode, 7-bit address
		movwf	SSPCON1
		bsf		SSPCON2,SEN					; enable clock stretching
		bsf		SSPCON3,BOEN				; SSPBUF is updated and NACK is generated
											; for a received address/data byte,
											; ignoring the state of SSPOV bit only if BF bit = 0.
		bsf		SSPCON3,SDAHT				; Minimum of 300 ns hold time
		movlw	I2C_ADDRESS					; load the slave address
		movwf	SSPADD
		banksel	PIR1
		bcf		PIR1,SSP1IF					; clear the SSP interrupt flag
		banksel	PIE1
		bsf		PIE1,SSP1IE					; enable SSP interrupts NOTE renamed
		banksel	PIR2
		bcf		PIR2,BCL1IF					; clear the SSP interrupt flag NOTE renamed
		banksel	PIE2
		bsf		PIE2,BCL1IE					; enable SSP interrupts NOTE rename
;set up the interrupt
        banksel INTCON
		bsf		INTCON,PEIE					; enable peripheral interrupts
		bsf		INTCON,GIE					; enable global interrupts
		return
;------------------------------------- END INITIALIZE ------------------------------------------
;
; The code below is taken directly from the Microchip AN734C and I have included
; the copyright and agreements
;
; Software License Agreement
;
; The software supplied herewith by Microchip Technology Incorporated
; (the 'Company') is intended and supplied to you, the Company's customer,
; for use solely and exclusively with products manufactured by the Company.
; The software is owned by the Company and/or its supplier, and is protected
; under applicable copyright laws. All rights are reserved. Any use in
; violation of the foregoing restrictions may subject the user to criminal
; sanctions under applicable laws, as well as to civil liability for the breach
; of the terms and conditions of this license.
;
; THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES,
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
; APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE
; LIABLE FOR SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON
; WHATSOEVER.

;-----------------------------------------------------------------------------------------------
;                                Interrupt Service Routine (ISR)
;-----------------------------------------------------------------------------------------------
READ
		banksel	SSPSTAT
		btfss	SSPSTAT,5					; was last byte an address or data?
		goto	R_ADDRESS					; if clear, it was an address
		goto	R_DATA						; if set, it was data
;
R_ADDRESS
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		movf	INDF1,W						; move value into W to load to SSP buffer
		banksel	SSPBUF
		movwf	SSPBUF						; load SSP buffer
		incf	INDEX_I2C,F					; increment INDEX_I2C 'pointer'
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
R_DATA
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		movf	INDF1,W						; move value into W to load to SSP buffer
		banksel	SSPBUF
		movwf	SSPBUF						; load SSP buffer
		incf	INDEX_I2C,F					; increment INDEX_I2C 'pointer'
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
WRITE
		banksel	SSPSTAT
		btfss	SSPSTAT,5					; was last byte an address or data?
		goto	W_ADDRESS					; if clear, it was an address
		goto	W_DATA						; if set, it was data

W_ADDRESS
        bsf		FIRST,0						; set bit 0 of FIRST
		banksel	SSPBUF
		movfw	SSPBUF						; move the contents of the buffer into W
                                            ; dummy read to clear the BF bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt

W_DATA
		btfss	FIRST,0						; is bit 0 set?
		goto	W_DATA_SUB					; if clear, go write data
		banksel	SSPBUF						; if set, first data byte is location to write to
		movfw	SSPBUF
		movwf	INDEX_I2C					; load INDEX_I2C with data address to write to
		clrf	FIRST						; clear the FIRST register
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt

W_DATA_SUB
		clrf	W                           ; clear W
		movlw	RX_ELEMENTS					; load array elements value
		banksel STATUS
		btfsc	STATUS,Z					; is Z clear?
		subwf	INDEX_I2C,W					; if Z = 1, subtract index from number of elements
		banksel	STATUS
		btfsc	STATUS,0					; did a carry occur after subtraction?
		goto	NO_MEM_OVERWRITE			; if so, Master is trying to write to many bytes
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		banksel	SSPBUF
		movfw	SSPBUF						; move the contents of the buffer into W
		movwf 	INDF1						; load INDF1 with data to write
		incf	INDEX_I2C,F					; increment 'pointer' to next memory location
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
NO_MEM_OVERWRITE
		banksel	SSPBUF
		movfw	SSPBUF						; move SSP buffer to W
                    						; clear buffer so no overwrite occurs
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
WRITE_COLL
		banksel	SSPCON1
		bcf		SSPCON1,WCOL				; clear WCOL bit
		banksel	SSPBUF
		movfw	SSPBUF						; move SSP buffer to W
                    						; clear SSP buffer
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
BUS_COLL
		banksel	SSPBUF
		clrf	SSPBUF						; clear the SSP buffer
		banksel	PIR2
		bcf		PIR2,BCL1IF					; clear the SSP interrupt flag
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		goto    EXIT						; Go to EXIT to return from interrupt
;
EXIT
        retfie                              ; Return from interrupt
;
;##############################################################################
		end									; END OF PROGRAM





